#!/usr/bin/env python

"""
SMI.py

This is the high-level interface into libsmi, defining a set of "shadow"
classes that provide an object-oriented interface to SMI data.

An instance of SMIClass represents the SMI parser, and its methods allow
access to the installed SMI database. 

"""

from _SMI import *
import sys, string, types
from UserList import UserList
from UserDict import UserDict


class SMIException(Exception):
	pass

class OID(UserList):
	def __init__(self):
		pass

class SMI_Path(UserList):
	def __init__(self):
		UserList.__init__(self)
		self.data.extend(string.split(smiGetPath(), ":"))

	def append(self, value):
		if type(value) is types.StringType:
			self.data.append(value)
			smiSetPath(string.join(self.data, ":"))
		else:
			raise TypeError

	def extend(self, aplist):
		if type(value) is types.ListType:
			self.data.extend(aplist)
			smiSetPath(string.join(self.data, ":"))
		else:
			raise TypeError

class ModuleDict(UserDict):
	def __init__(self):
		UserDict.__init__(self)

class SMIModule(SmiModule):
	def __init__(self):
		SmiModule.__init__(self)


class SMINode(SmiNode):
	def __init__(self):
		SmiNode.__init__(self)


class SMIValue(SmiValue):
	def __init__(self):
		SmiValue.__init__(self)


class SMIClass:
	def __init__(self, tag="NULL"):
		if smiInit(tag) != 0:
			raise SMIException
		self.path = SMI_Path()
		self.modules = ModuleDict()
		# pre-load some base modules
		self.loadModules(["SNMPv2-SMI", "SNMPv2-TC", "SNMPv2-CONF"])

	def __del__(self):
		smiExit()

# This will automatically expose objects in the _SMI module as
# objects/attributes/methods of this class.
	def __getattr__(self, attrname):
		if sys.modules["_SMI"].__dict__.has_key(attrname):
			return sys.modules["_SMI"].__dict__[attrname]
		else:
			raise AttributeError, "No SMI object "+attrname

	def getModule(self, modname):
		smimod = smiGetModule(modname)
		if smimod is not None:
			self.modules[smimod.name] = smimod
		return smimod

	def loadModules(self, modlist):
		if type(modlist) is types.ListType:
			for mod in modlist:
				smiLoadModule(mod)
		elif type(modlist) is types.StringType:
			smiLoadModule(modlist)
		self._getModules()

	def _getModules(self):
		mod = smiGetFirstModule()
		while mod is not None:
			self.modules[mod.name] = mod
			mod = smiGetNextModule(mod)




if __name__ == "__main__":

	import os
	smi = SMIClass()

	smi.loadModules(["IP-MIB", "TCP-MIB"])

	# XXX search methods need to be added
	fn = smi.smiGetFirstNode(smi.modules["IP-MIB"], smi.SMI_NODEKIND_ANY)
	nn = smi.smiGetNextNode(fn, smi.SMI_NODEKIND_ANY)
	print nn.name
	print nn.oid
	print get_SmiNode_oid(nn)
	del smi


